#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>

#define NUM_BUTTONS 16

uint8_t button_pin[NUM_BUTTONS] = { 0,  5, 10, A0,
                                    2,  7, 14, A1,
                                    3,  8, 15, A2,
                                    4,  9, 16, A3};


uint8_t button_ctr[NUM_BUTTONS] = {0};
uint8_t show_type[NUM_BUTTONS] = {0};

#define MATRIX_PIN    6    // Digital IO pin connected to the NeoPixels.
#define BUZZER_PIN    1    // Digital IO pin connected to the buzzer.

// MATRIX DECLARATION:
// Parameter 1 = width of NeoPixel matrix
// Parameter 2 = height of matrix
// Parameter 3 = pin number (most are valid)
// Parameter 4 = matrix layout flags, add together as needed:
//   NEO_MATRIX_TOP, NEO_MATRIX_BOTTOM, NEO_MATRIX_LEFT, NEO_MATRIX_RIGHT:
//     Position of the FIRST LED in the matrix; pick two, e.g.
//     NEO_MATRIX_TOP + NEO_MATRIX_LEFT for the top-left corner.
//   NEO_MATRIX_ROWS, NEO_MATRIX_COLUMNS: LEDs are arranged in horizontal
//     rows or in vertical columns, respectively; pick one or the other.
//   NEO_MATRIX_PROGRESSIVE, NEO_MATRIX_ZIGZAG: all rows/columns proceed
//     in the same order, or alternate lines reverse direction; pick one.
//   See example below for these values in action.
// Parameter 5 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)

Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(16, 4, MATRIX_PIN,
  NEO_MATRIX_TOP     + NEO_MATRIX_LEFT +
  NEO_MATRIX_ROWS    + NEO_MATRIX_ZIGZAG,
  NEO_GRB            + NEO_KHZ800);

void setup() {
  for (uint8_t i = 0; i < NUM_BUTTONS; i++) {
      pinMode(button_pin[i], INPUT_PULLUP);
  }
  matrix.begin();
  matrix.fillScreen(0);
  //matrix.setBrightness(40); 
  matrix.setBrightness(128);
  matrix.show();
}

#define TRG_FIRST 2  // loops
#define TRG_DELAY 20 // loops
#define LOOP_TIME 20 // ms

void loop() {
  // Short delay to debounce button.
  delay(LOOP_TIME);

  for (uint8_t i = 0; i < NUM_BUTTONS; i++) {
    
    if (digitalRead(button_pin[i]) == LOW) {
      button_ctr[i]++;
    }
    else {
      button_ctr[i] = 0;
    }

    if (button_ctr[i] % TRG_DELAY == TRG_FIRST) {
        show_type[i]++;
        if (show_type[i] > 9)
          show_type[i] = 0;
        set_color(i, show_type[i]);  // set button according to show type here
    
        if (button_ctr[i] == TRG_DELAY + TRG_FIRST) {
          button_ctr[i] = TRG_FIRST;
        }
      }
    
  }
  matrix.show();
}

const uint16_t show_colors[10] = {matrix.Color(0, 0, 0),
                                  matrix.Color(255, 0, 0),
                                  matrix.Color(255, 75, 0),
                                  matrix.Color(255, 150, 0),
                                  matrix.Color(0, 255, 0),
                                  matrix.Color(0, 255, 255),
                                  matrix.Color(0, 0, 255),
                                  matrix.Color(128, 0, 255),
                                  matrix.Color(255, 0, 255),
                                  matrix.Color(255, 255, 255)};

void set_color(uint8_t pos, uint8_t color)
{
  uint8_t x, y;
  x = (pos % 4) * 4;
  y =  pos / 4;

  for (uint8_t i = 0; i < 4; i++) {
    matrix.drawPixel(x + i, y, show_colors[color]);
  }
}
